"""Main module."""

__all__ = [
    'INTENSITY',
    'LABEL',
    'SAMPLING_MAP',
    'DATA',
    'AFFINE',
    'IMAGE',
    'LOCATION',
]

INTENSITY = 'intensity'
LABEL = 'label'
SAMPLING_MAP = 'sampling_map'

DATA = 'data'
AFFINE = 'affine'

# For aggregator
IMAGE = 'image'
LOCATION = 'location'
